/// Binarize allows for a more streamlined and extendable binary creation experience.
library binarize;

import 'dart:typed_data';
import 'package:binarize/src/payload_type.dart';

export 'src/payload_type.dart';

part 'src/payload.dart';

Uint8List binarize(_PayloadWriter payload) {
  final length = payload._data.fold<int>(
    0,
    (v, d) => v + (d['type'].length(d['value'])) as int,
  );

  final uint8List = Uint8List(length);
  final byteData = ByteData.view(uint8List.buffer);

  var offset = 0;
  for (final data in payload._data) {
    final value = data['value'];
    final type = data['type'] as PayloadType;

    type.set(value, byteData, offset);
    offset += type.length(value);
  }

  return uint8List;
}
