part of binarize;

abstract class Payload {
  /// Returns the length of the payload in bytes.
  int get length;

  /// Create a writeable payload.
  static _PayloadWriter write() {
    return _PayloadWriter();
  }

  /// Create a readdable payload.
  static _PayloadReader read(Iterable<int> data) {
    return _PayloadReader(Uint8List.fromList(data.toList()));
  }
}

class _PayloadWriter extends Payload {
  final _data = <Map<String, dynamic>>[];

  @override
  int get length => binarize(this).lengthInBytes;

  /// Set the given [value] using the given [PayloadType].
  void set<T>(PayloadType<T> type, T value) {
    _data.add({'value': value, 'type': type});
  }
}

class _PayloadReader extends Payload {
  final ByteData _byteData;
  var _offset = 0;

  @override
  int get length => _byteData.lengthInBytes;

  _PayloadReader(Uint8List payload) : _byteData = ByteData.view(payload.buffer);

  /// Get the next value by using the given [PayloadType].
  T get<T>(PayloadType<T> type) {
    final value = type.get(_byteData, _offset);
    _offset += type.length(value);
    return value;
  }
}
