import 'dart:typed_data';

export 'types/boolean.dart';
export 'types/flags.dart';
export 'types/float32.dart';
export 'types/float64.dart';
export 'types/int8.dart';
export 'types/int16.dart';
export 'types/int32.dart';
export 'types/int64.dart';
export 'types/string.dart';
export 'types/uint8.dart';
export 'types/uint16.dart';
export 'types/uint32.dart';
export 'types/uint64.dart';

abstract class PayloadType<T> {
  const PayloadType();

  /// Tester method to see if given value is even allowed for this [PayloadType].
  bool isType(dynamic value) => value is T;

  /// Returns the byte length of the [PayloadType] with the given [value].
  ///
  /// Will be used to calculate the next offset when setting and getting data.
  int length(T value);

  /// Retrieve the value from the [data] at the given [offset].
  T get(ByteData data, int offset);

  /// Set the [value] in the [data] at the given [offset].
  void set(T value, ByteData data, int offset);
}
